@@ -233,7 +233,8 @@ def summary(model: nn.Module, sample_batch: tensor_dict_type) -> None:
             m_dict["num_trainable_params"] = num_trainable_params_
             raw_summary_dict[m_name] = m_dict
 
-        hooks.append(module.register_forward_hook(hook))
+        if not isinstance(module, torch.jit.ScriptModule):
+            hooks.append(module.register_forward_hook(hook))
 
     # get names
     def _inject_names(m: nn.Module, previous_names: List[str]) -> None:
@@ -258,12 +259,23 @@ def summary(model: nn.Module, sample_batch: tensor_dict_type) -> None:
             _inject_names(child, current_names)
 
     module_names: OrderedDict[nn.Module, str] = OrderedDict()
+    existing_names: Set[str] = set()
+
+    def _get_name(original: str) -> str:
+        count = 0
+        final_name = original
+        while final_name in existing_names:
+            count += 1
+            final_name = f"{original}_{count}"
+        existing_names.add(final_name)
+        return final_name
+
     for extractor in model.extractors.values():  # type: ignore
-        extractor_name = type(extractor).__name__
+        extractor_name = _get_name(type(extractor).__name__)
         module_names[extractor] = extractor_name
         _inject_names(extractor, [extractor_name])
     for head in model.heads.values():  # type: ignore
-        head_name = type(head).__name__
+        head_name = _get_name(type(head).__name__)
         module_names[head] = head_name
         _inject_names(head, [head_name])
 
@@ -494,6 +506,10 @@ class Initializer(LoggingMixinWithRank):
             param.data.copy_(weight_base.reshape(param.shape))
             param.data.mul_(std).add_(mean)
 
+    def orthogonal(self, param: param_type) -> None:
+        gain = self.config.setdefault("gain", 1.0)
+        nn.init.orthogonal_(param.data, gain)
+
 
 class _multiplied_activation(nn.Module, metaclass=ABCMeta):
     def __init__(
