@@ -311,9 +311,10 @@ class ModelBase(ModelProtocol, metaclass=ABCMeta):
             extractor = self.extractors[extractor_unique_key]
         else:
             if extractor_exists:
-                new_index = 1
+                new_index = 0
                 new_extractor_unique_key = extractor_unique_key
                 while new_extractor_unique_key in self.extractors:
+                    new_index += 1
                     new_extractor_unique_key = f"{extractor_unique_key}_{new_index}"
                 extractor_unique_key = new_extractor_unique_key
             if extractor_cfg_key in self._extractor_configs:
@@ -411,7 +412,6 @@ class ModelBase(ModelProtocol, metaclass=ABCMeta):
         loader_name: Optional[str] = None,
         **kwargs: Any,
     ) -> tensor_dict_type:
-        # batch will have `categorical`, `numerical` and `labels` keys
         x_batch = batch["x_batch"]
         split = self._split_features(x_batch, batch_indices, loader_name)
         outputs = self.execute(split)
@@ -454,6 +454,34 @@ class ModelBase(ModelProtocol, metaclass=ABCMeta):
             enable_timing=self.timing,
         )
 
+    def _transform(
+        self,
+        transform: Transform,
+        net: Union[Tensor, SplitFeatures],
+    ) -> Tensor:
+        return net if isinstance(net, Tensor) else transform(net)
+
+    def _extract(
+        self,
+        extractor: ExtractorBase,
+        transformed: Tensor,
+        extract_kwargs: Dict[str, Any],
+    ) -> Tensor:
+        extracted = extractor(transformed, **extract_kwargs)
+        extracted_shape = extracted.shape
+        if extractor.flatten_ts:
+            if len(extracted_shape) == 3:
+                extracted = extracted.view(extracted_shape[0], -1)
+        return extracted
+
+    def _head(
+        self,
+        head: HeadBase,
+        extracted: Tensor,
+        head_kwargs: Dict[str, Any],
+    ) -> Tensor:
+        return head(extracted, **head_kwargs)
+
     def execute(
         self,
         net: Union[Tensor, SplitFeatures],
@@ -469,28 +497,24 @@ class ModelBase(ModelProtocol, metaclass=ABCMeta):
             # transform
             transformed = self._transform_cache.get(transform_key)
             if transformed is None:
-                transform = self.transforms[transform_key]
-                transformed = net if isinstance(net, Tensor) else transform(net)
+                transformed = self._transform(self.transforms[transform_key], net)
                 self._transform_cache[transform_key] = transformed
             # extract
             if extract_kwargs_dict is None:
                 extract_kwargs_dict = {}
             extracted = self._extractor_cache.get(extractor_key)
             if extracted is None:
-                extractor = self.extractors[extractor_key]
                 extract_kwargs = extract_kwargs_dict.get(extractor_key, {})
-                extracted = extractor(transformed, **extract_kwargs)
-                extracted_shape = extracted.shape
-                if extractor.flatten_ts:
-                    if len(extracted_shape) == 3:
-                        extracted = extracted.view(extracted_shape[0], -1)
+                extracted = self._extract(
+                    self.extractors[extractor_key],
+                    transformed,
+                    extract_kwargs,
+                )
                 self._extractor_cache[extractor_key] = extracted
-            # execute
             if head_kwargs_dict is None:
                 head_kwargs_dict = {}
             head_kwargs = head_kwargs_dict.get(key, {})
-            results[key] = self.heads[key](extracted, **head_kwargs)
-        # finalize
+            results[key] = self._head(self.heads[key], extracted, head_kwargs)
         if clear_cache:
             self.clear_execute_cache()
         return results
@@ -585,4 +609,48 @@ class ModelBase(ModelProtocol, metaclass=ABCMeta):
         return _core
 
 
-__all__ = ["ModelBase"]
+class SiameseModelBase(ModelBase):
+    def _extract(
+        self,
+        extractor: ExtractorBase,
+        transformed: Tensor,
+        extract_kwargs: Dict[str, Any],
+    ) -> Tensor:
+        if not self.training:
+            return super()._extract(extractor, transformed, extract_kwargs)
+        num_slice = transformed.shape[0] // 2
+        t1, t2 = transformed[:num_slice], transformed[num_slice : 2 * num_slice]
+        e1 = super()._extract(extractor, t1, shallow_copy_dict(extract_kwargs))
+        e2 = super()._extract(extractor, t2, shallow_copy_dict(extract_kwargs))
+        return e2 - e1
+
+    def forward(
+        self,
+        batch: tensor_dict_type,
+        batch_idx: Optional[int] = None,
+        state: Optional[TrainerState] = None,
+        batch_indices: Optional[np.ndarray] = None,
+        loader_name: Optional[str] = None,
+        **kwargs: Any,
+    ) -> tensor_dict_type:
+        results = super().forward(
+            batch,
+            batch_idx,
+            state,
+            batch_indices,
+            loader_name,
+            **kwargs,
+        )
+        if not self.training:
+            return results
+        labels = batch["labels"]
+        num_slice = labels.shape[0] // 2
+        l1, l2 = labels[:num_slice], labels[num_slice : 2 * num_slice]
+        batch["labels"] = l2 - l1
+        return results
+
+
+__all__ = [
+    "ModelBase",
+    "SiameseModelBase",
+]
