@@ -1,4 +1,5 @@
 from .base import ModelBase
+from .base import SiameseModelBase
 
 
 @ModelBase.register("rnn")
@@ -7,6 +8,12 @@ class RNN(ModelBase):
     pass
 
 
+@SiameseModelBase.register("siamese_rnn")
+@SiameseModelBase.register_pipe("rnn", head="fcnn")
+class SiameseRNN(SiameseModelBase):
+    pass
+
+
 @ModelBase.register("tree_rnn")
 @ModelBase.register_pipe("rnn_dndf", extractor="rnn", head="dndf")
 @ModelBase.register_pipe("rnn_fcnn", extractor="rnn", head="fcnn", head_config="pruned")
@@ -14,4 +21,8 @@ class TreeRNN(ModelBase):
     pass
 
 
-__all__ = ["RNN"]
+__all__ = [
+    "RNN",
+    "TreeRNN",
+    "SiameseRNN",
+]
