@@ -30,6 +30,9 @@ from ..misc.toolkit import *
 from ..types import tensor_tuple_type
 
 
+mapping_dict: Dict[str, Type["MappingBase"]] = {}
+
+
 class Linear(Module):
     def __init__(
         self,
@@ -79,7 +82,18 @@ class Linear(Module):
             self.linear.bias.data.fill_(bias_fill)
 
 
-class Mapping(Module):
+class MappingBase(Module):
+    def __init__(self, *args: Any, **kwargs: Any):
+        super().__init__()
+
+    @classmethod
+    def register(cls, name: str) -> Callable[[Type], Type]:
+        global mapping_dict
+        return register_core(name, mapping_dict)
+
+
+@MappingBase.register("basic")
+class Mapping(MappingBase):
     def __init__(
         self,
         in_dim: int,
@@ -160,7 +174,7 @@ class Mapping(Module):
         )
 
 
-class ResBlock(Module):
+class _SkipConnectBlock(MappingBase, metaclass=ABCMeta):
     def __init__(
         self,
         in_dim: int,
@@ -175,13 +189,15 @@ class ResBlock(Module):
         **kwargs: Any,
     ):
         super().__init__()
-        self.in_bn = None if not batch_norm else BN(in_dim)
-        if activation is None:
-            self.in_activation: Optional[Module] = None
-        else:
-            activation_config = kwargs.setdefault("activation_config", None)
-            self.in_activation = Activations.make(activation, activation_config)
-        self.mapping = Mapping(
+        self.linear_mapping = Linear(
+            in_dim,
+            latent_dim,
+            bias=True if bias is None else bias,
+            pruner_config=pruner_config,
+            init_method=init_method,
+            **kwargs,
+        )
+        self.nonlinear_mapping = Mapping(
             in_dim,
             latent_dim,
             bias=bias,
@@ -192,34 +208,108 @@ class ResBlock(Module):
             init_method=init_method,
             **kwargs,
         )
-        if bias is None:
-            bias = True
-        self.linear = Linear(
-            latent_dim,
-            latent_dim,
-            bias=bias,
-            pruner_config=pruner_config,
-            init_method=init_method,
-            **kwargs,
-        )
-        self.res_linear = Linear(
+
+    def forward(self, net: Tensor) -> Tensor:
+        linear = self.linear_mapping(net)
+        nonlinear = self.nonlinear_mapping(net)
+        return self._skip_connect(net, linear, nonlinear)
+
+    @abstractmethod
+    def _skip_connect(self, net: Tensor, linear: Tensor, nonlinear: Tensor) -> Tensor:
+        pass
+
+
+@MappingBase.register("res")
+class ResBlock(_SkipConnectBlock):
+    def __init__(
+        self,
+        in_dim: int,
+        latent_dim: int,
+        *,
+        bias: Optional[bool] = None,
+        pruner_config: Optional[dict] = None,
+        dropout: float = 0.0,
+        batch_norm: bool = True,
+        activation: Optional[str] = "ReLU",
+        init_method: str = "xavier_uniform",
+        **kwargs: Any,
+    ):
+        super().__init__(
             in_dim,
             latent_dim,
             bias=bias,
             pruner_config=pruner_config,
+            dropout=dropout,
+            batch_norm=batch_norm,
+            activation=activation,
             init_method=init_method,
             **kwargs,
         )
+        self.res_linear = Linear(
+            latent_dim,
+            latent_dim,
+            bias=True if bias is None else bias,
+            pruner_config=pruner_config,
+            init_method=init_method,
+            **kwargs,
+        )
+        self.res_bn = None if not batch_norm else BN(latent_dim)
+        self.res_dropout = Dropout(dropout)
+        if activation is None:
+            self.res_activation: Optional[Module] = None
+        else:
+            activation_config = kwargs.setdefault("activation_config", None)
+            self.res_activation = Activations.make(activation, activation_config)
 
-    def forward(self, net: Tensor) -> Tensor:
-        latent = net
-        if self.in_bn is not None:
-            latent = self.in_bn(latent)
-        if self.in_activation is not None:
-            latent = self.in_activation(latent)
-        latent = self.mapping(latent)
-        latent = self.linear(latent)
-        return self.res_linear(net) + latent
+    def _skip_connect(self, net: Tensor, linear: Tensor, nonlinear: Tensor) -> Tensor:
+        net = linear + self.res_linear(nonlinear)
+        if self.res_bn is not None:
+            net = self.res_bn(net)
+        net = self.res_dropout(net)
+        if self.res_activation is None:
+            return net
+        return self.res_activation(net)
+
+
+@MappingBase.register("highway")
+class HighwayBlock(_SkipConnectBlock):
+    def __init__(
+        self,
+        in_dim: int,
+        latent_dim: int,
+        *,
+        bias: Optional[bool] = None,
+        pruner_config: Optional[dict] = None,
+        dropout: float = 0.0,
+        batch_norm: bool = True,
+        activation: Optional[str] = "ReLU",
+        init_method: str = "xavier_uniform",
+        **kwargs: Any,
+    ):
+        super().__init__(
+            in_dim,
+            latent_dim,
+            bias=bias,
+            pruner_config=pruner_config,
+            dropout=dropout,
+            batch_norm=batch_norm,
+            activation=activation,
+            init_method=init_method,
+            **kwargs,
+        )
+        self.gate_linear = Linear(
+            in_dim,
+            latent_dim,
+            bias=True if bias is None else bias,
+            pruner_config=pruner_config,
+            init_method=init_method,
+            **kwargs,
+        )
+        self.sigmoid = nn.Sigmoid()
+
+    def _skip_connect(self, net: Tensor, linear: Tensor, nonlinear: Tensor) -> Tensor:
+        gate = self.sigmoid(self.gate_linear(net))
+        return gate * nonlinear + (1.0 - gate) * linear
 
 
 class MLP(Module):
@@ -230,14 +320,16 @@ class MLP(Module):
         num_units: List[int],
         mapping_configs: Union[Dict[str, Any], List[Dict[str, Any]]],
         *,
+        mapping_type: str = "basic",
         final_mapping_config: Optional[Dict[str, Any]] = None,
     ):
         super().__init__()
-        mappings: List[Union[Linear, Mapping]] = []
+        mappings: List[Module] = []
         if isinstance(mapping_configs, dict):
             mapping_configs = [mapping_configs] * len(num_units)
+        mapping_base = mapping_dict[mapping_type]
         for num_unit, mapping_config in zip(num_units, mapping_configs):
-            mappings.append(Mapping(in_dim, num_unit, **mapping_config))
+            mappings.append(mapping_base(in_dim, num_unit, **mapping_config))
             in_dim = num_unit
         if out_dim is not None:
             if final_mapping_config is None:
@@ -362,49 +454,6 @@ class MLP(Module):
             final_mapping_config=final_mapping_config,
         )
 
-    @classmethod
-    def res_funnel(
-        cls,
-        in_dim: int,
-        out_dim: int,
-        max_dim: int,
-        num_layers: int,
-        *,
-        bias: bool = True,
-        max_dropout: float = 0.1,
-        batch_norm: bool = True,
-        activation: Optional[str] = "relu",
-        pruner_config: Optional[Dict[str, Any]] = None,
-    ) -> nn.Sequential:
-        dropouts, num_units = cls.get_funnel_settings(
-            max_dropout,
-            max_dim,
-            out_dim,
-            num_layers,
-        )
-        in_linear = Linear(in_dim, max_dim, bias=bias, pruner_config=pruner_config)
-        blocks: List[Module] = [in_linear]
-        dim = max_dim
-        for dropout, num_unit in zip(dropouts[1:], num_units[1:]):
-            blocks.append(
-                ResBlock(
-                    dim,
-                    num_unit,
-                    bias=bias,
-                    pruner_config=pruner_config,
-                    dropout=dropout,
-                    batch_norm=batch_norm,
-                    activation=activation,
-                )
-            )
-            dim = num_unit
-
-        blocks.append(BN(dim))
-        if activation is not None:
-            blocks.append(Activations.make(activation))
-        blocks.append(Linear(dim, out_dim, bias=bias, pruner_config=pruner_config))
-        return nn.Sequential(*blocks)
-
 
 class LeafAggregation(torch.autograd.Function):
     @staticmethod
@@ -501,7 +550,7 @@ class DNDF(Module):
     def __init__(
         self,
         in_dim: int,
-        out_dim: int,
+        out_dim: Optional[int],
         *,
         num_tree: int = 10,
         tree_depth: int = 4,
@@ -512,7 +561,9 @@ class DNDF(Module):
         super().__init__()
         self._num_tree = num_tree
         self._tree_depth = tree_depth
-        self._is_regression = out_dim == 1 if is_regression is None else is_regression
+        self._is_regression = is_regression
+        if out_dim is not None and is_regression is None:
+            self._is_regression = out_dim == 1
         self._num_leaf = 2 ** (self._tree_depth + 1)
         self._num_internals = self._num_leaf - 1
         self._output_dim = out_dim
@@ -525,10 +576,13 @@ class DNDF(Module):
             self._num_internals * self._num_tree,
             **tree_proj_config,
         )
-        leaves_shape = self._num_tree * self._num_leaf, self._output_dim
-        self.leaves = nn.Parameter(torch.empty(*leaves_shape))
-        with torch.no_grad():
-            torch.nn.init.xavier_uniform_(self.leaves.data)
+        if out_dim is None:
+            self.leaves = None
+        else:
+            leaves_shape = self._num_tree * self._num_leaf, out_dim
+            self.leaves = nn.Parameter(torch.empty(*leaves_shape))
+            with torch.no_grad():
+                torch.nn.init.xavier_uniform_(self.leaves.data)
         # buffers
         num_repeat, num_local_internals = self._num_leaf // 2, 1
         ones_np = np.repeat([1, -1], num_repeat)
@@ -588,6 +642,8 @@ class DNDF(Module):
                 routes *= flat_probabilities.take(current_indices)
 
         features = routes.transpose(0, 1).contiguous().view(num_batch, -1)
+        if self.leaves is None or self._output_dim is None:
+            return features.view(num_batch, self._num_tree, -1)
         if self._is_regression or self._output_dim <= 1:
             outputs = features.mm(self.leaves)
         else:
