@@ -230,8 +230,6 @@ class Transformer(ExtractorBase):
         self.scaling = float(latent_dim) ** 0.5
         self.latent_dim = latent_dim
         self.input_linear = Linear(self.in_dim, latent_dim, **input_linear_config)
-        # head token
-        self.head_token = nn.Parameter(torch.randn(1, 1, latent_dim))
         # position encoding
         self.position_encoding = PositionalEncoding(latent_dim, seq_len, dropout)
         # transformer blocks
@@ -239,7 +237,7 @@ class Transformer(ExtractorBase):
         layer_config["norm_type"] = norm_type
         layer_config["attention_type"] = attention_type
         if attention_type == "decayed":
-            layer_config["seq_len"] = seq_len + 1
+            layer_config["seq_len"] = seq_len
         layer = TransformerLayer(latent_dim, num_heads, **layer_config)
         encoder_base = TransformerEncoder.get(encoder_type)
         self.encoder = encoder_base(layer, num_layers, dimensions, **encoder_config)
@@ -251,27 +249,19 @@ class Transformer(ExtractorBase):
 
     @property
     def out_dim(self) -> int:
-        return 2 * self.latent_dim
+        return self.latent_dim
 
     def _aggregate(self, net: Tensor) -> Tensor:
-        no_head_token = net[..., :-1, :]
-        a_hat = self.final_attn_linear(no_head_token)
+        a_hat = self.final_attn_linear(net)
         a_prob = F.softmax(a_hat, dim=1)
-        a = torch.sum(a_prob * no_head_token, dim=1)
-        return torch.cat([a, net[..., -1, :]], 1)
+        return torch.sum(a_prob * net, dim=1)
 
     def forward(self, net: Tensor) -> Tensor:
-        # input -> latent
         net = self.input_linear(net)
         net = torch.tanh(net)
         net = net * self.scaling
         net = self.position_encoding(net)
-        # concat head token
-        expanded_token = self.head_token.expand(net.shape[0], 1, self.latent_dim)
-        net = torch.cat([net, expanded_token], dim=1)
-        # encode latent vector with transformer
         net = self.encoder(net, None)
-        # aggregate
         return self._aggregate(net)
 
 
