@@ -224,7 +224,6 @@ class SamplerProtocol(ABC):
 
 class DataLoaderProtocol(ABC):
     is_onnx: bool = False
-    _num_siamese: int = 1
     labels_key: str = "labels"
 
     data: DataProtocol
@@ -308,7 +307,6 @@ class PrefetchLoader:
         self.next_batch_indices: Optional[torch.Tensor]
         self.stop_at_next_batch = False
         self.batch_size = loader.batch_size
-        self._num_siamese = loader._num_siamese
 
     def __len__(self) -> int:
         return len(self.loader)
@@ -576,7 +574,7 @@ class ModelProtocol(nn.Module, LoggingMixinWithRank, metaclass=ABCMeta):
             if param.requires_grad:
                 msg += name + "\n"
         msg += "\n".join(["-" * 100, "=" * 100, "buffers", "-" * 100, ""])
-        for name, param in self.named_buffers():
+        for name, _ in self.named_buffers():
             msg += name + "\n"
         msg += "\n".join(
             ["-" * 100, "=" * 100, "structure", "-" * 100, str(self), "-" * 100, ""]
